"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

DOM debugging allows setting breakpoints on particular DOM operations and events. JavaScript
execution will stop on these operations as if there was a regular breakpoint set.
https://chromedevtools.github.io/devtools-protocol/tot/DOMDebugger/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOM,
    DOMDebugger,
    Runtime,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class GetEventListenersInput(InputModel):

    objectId: Runtime.RemoteObjectId
    depth: int | None = None
    pierce: bool | None = None


class GetEventListenersOutput(OutputModel):

    listeners: list[DOMDebugger.EventListener]


class GetEventListeners(CDPMethod[GetEventListenersOutput]):
    """ Returns event listeners of the given object. """

    INPUT_VALIDATOR = GetEventListenersInput
    OUTPUT_VALIDATOR = GetEventListenersOutput

    def __init__(
        self,
        *,
        object_id: Runtime.RemoteObjectId,
        depth: int | None = None,
        pierce: bool | None = None
    ):
        super().__init__(
            objectId=object_id,
            depth=depth,
            pierce=pierce
        )


class RemoveDOMBreakpointInput(InputModel):

    nodeId: DOM.NodeId
    type: DOMDebugger.DOMBreakpointType


class RemoveDOMBreakpoint(CDPMethod[None]):
    """ Removes DOM breakpoint that was set using `setDOMBreakpoint`. """

    INPUT_VALIDATOR = RemoveDOMBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        type_: DOMDebugger.DOMBreakpointType
    ):
        super().__init__(
            nodeId=node_id,
            type=type_
        )


class RemoveEventListenerBreakpointInput(InputModel):

    eventName: str
    targetName: str | None = None  # experimental


class RemoveEventListenerBreakpoint(CDPMethod[None]):
    """ Removes breakpoint on particular DOM event. """

    INPUT_VALIDATOR = RemoveEventListenerBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_name: str,
        target_name: str | None = None
    ):
        super().__init__(
            eventName=event_name,
            targetName=target_name
        )


class RemoveInstrumentationBreakpointInput(InputModel):

    eventName: str


class RemoveInstrumentationBreakpoint(CDPMethod[None]):  # experimental deprecated
    """ Removes breakpoint on particular native event. """

    INPUT_VALIDATOR = RemoveInstrumentationBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_name: str
    ):
        super().__init__(
            eventName=event_name
        )


class RemoveXHRBreakpointInput(InputModel):

    url: str


class RemoveXHRBreakpoint(CDPMethod[None]):
    """ Removes breakpoint from XMLHttpRequest. """

    INPUT_VALIDATOR = RemoveXHRBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        url: str
    ):
        super().__init__(
            url=url
        )


class SetBreakOnCSPViolationInput(InputModel):

    violationTypes: list[DOMDebugger.CSPViolationType]


class SetBreakOnCSPViolation(CDPMethod[None]):  # experimental
    """ Sets breakpoint on particular CSP violations. """

    INPUT_VALIDATOR = SetBreakOnCSPViolationInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        violation_types: list[DOMDebugger.CSPViolationType]
    ):
        super().__init__(
            violationTypes=violation_types
        )


class SetDOMBreakpointInput(InputModel):

    nodeId: DOM.NodeId
    type: DOMDebugger.DOMBreakpointType


class SetDOMBreakpoint(CDPMethod[None]):
    """ Sets breakpoint on particular operation with DOM. """

    INPUT_VALIDATOR = SetDOMBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        type_: DOMDebugger.DOMBreakpointType
    ):
        super().__init__(
            nodeId=node_id,
            type=type_
        )


class SetEventListenerBreakpointInput(InputModel):

    eventName: str
    targetName: str | None = None  # experimental


class SetEventListenerBreakpoint(CDPMethod[None]):
    """ Sets breakpoint on particular DOM event. """

    INPUT_VALIDATOR = SetEventListenerBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_name: str,
        target_name: str | None = None
    ):
        super().__init__(
            eventName=event_name,
            targetName=target_name
        )


class SetInstrumentationBreakpointInput(InputModel):

    eventName: str


class SetInstrumentationBreakpoint(CDPMethod[None]):  # experimental deprecated
    """ Sets breakpoint on particular native event. """

    INPUT_VALIDATOR = SetInstrumentationBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_name: str
    ):
        super().__init__(
            eventName=event_name
        )


class SetXHRBreakpointInput(InputModel):

    url: str


class SetXHRBreakpoint(CDPMethod[None]):
    """ Sets breakpoint on XMLHttpRequest. """

    INPUT_VALIDATOR = SetXHRBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        url: str
    ):
        super().__init__(
            url=url
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

DOM debugging allows setting breakpoints on particular DOM operations and events. JavaScript
execution will stop on these operations as if there was a regular breakpoint set.
https://chromedevtools.github.io/devtools-protocol/tot/DOMDebugger/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOMDebuggerCSPViolationType,
    DOMDebuggerDOMBreakpointType,
    DOMDebuggerEventListener,
)

DOMBreakpointType = DOMDebuggerDOMBreakpointType
CSPViolationType = DOMDebuggerCSPViolationType
EventListener = DOMDebuggerEventListener



from .events import AuthRequired, RequestPaused
from .methods import (
    ContinueRequest,
    ContinueResponse,
    ContinueWithAuth,
    Disable,
    Enable,
    FailRequest,
    FulfillRequest,
    GetResponseBody,
    TakeResponseBodyAsStream,
)
from .types import AuthChallenge, AuthChallengeResponse, HeaderEntry, RequestId, RequestPattern, RequestStage

__all__ = [
    RequestId,
    RequestStage,
    RequestPattern,
    HeaderEntry,
    AuthChallenge,
    AuthChallengeResponse,
    RequestPaused,
    AuthRequired,
    Disable,
    Enable,
    FailRequest,
    FulfillRequest,
    ContinueRequest,
    ContinueWithAuth,
    ContinueResponse,
    GetResponseBody,
    TakeResponseBodyAsStream,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Security/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Security,
)
from cdpkit.protocol.base import CDPMethod, InputModel


class Disable(CDPMethod[None]):
    """ Disables tracking security state changes. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables tracking security state changes. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class SetIgnoreCertificateErrorsInput(InputModel):

    ignore: bool


class SetIgnoreCertificateErrors(CDPMethod[None]):
    """ Enable/disable whether all certificate errors should be ignored. """

    INPUT_VALIDATOR = SetIgnoreCertificateErrorsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        ignore: bool
    ):
        super().__init__(
            ignore=ignore
        )


class HandleCertificateErrorInput(InputModel):

    eventId: int
    action: Security.CertificateErrorAction


class HandleCertificateError(CDPMethod[None]):  # deprecated
    """ Handles a certificate error that fired a certificateError event. """

    INPUT_VALIDATOR = HandleCertificateErrorInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_id: int,
        action: Security.CertificateErrorAction
    ):
        super().__init__(
            eventId=event_id,
            action=action
        )


class SetOverrideCertificateErrorsInput(InputModel):

    override: bool


class SetOverrideCertificateErrors(CDPMethod[None]):  # deprecated
    """ Enable/disable overriding certificate errors. If enabled, all certificate error events need to
    be handled by the DevTools client and should be answered with `handleCertificateError` commands. """

    INPUT_VALIDATOR = SetOverrideCertificateErrorsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        override: bool
    ):
        super().__init__(
            override=override
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Input/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    InputDragData,
    InputDragDataItem,
    InputGestureSourceType,
    InputMouseButton,
    InputTimeSinceEpoch,
    InputTouchPoint,
)

TouchPoint = InputTouchPoint
GestureSourceType = InputGestureSourceType
MouseButton = InputMouseButton
TimeSinceEpoch = InputTimeSinceEpoch
DragDataItem = InputDragDataItem
DragData = InputDragData



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/FileSystem/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    FileSystem,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class GetDirectoryInput(InputModel):

    bucketFileSystemLocator: FileSystem.BucketFileSystemLocator


class GetDirectoryOutput(OutputModel):

    directory: FileSystem.Directory


class GetDirectory(CDPMethod[GetDirectoryOutput]):

    INPUT_VALIDATOR = GetDirectoryInput
    OUTPUT_VALIDATOR = GetDirectoryOutput

    def __init__(
        self,
        *,
        bucket_file_system_locator: FileSystem.BucketFileSystemLocator
    ):
        super().__init__(
            bucketFileSystemLocator=bucket_file_system_locator
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Animation/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Animation,
)
from cdpkit.protocol.base import CDPEvent


class AnimationCanceled(CDPEvent):
    """ Event for when an animation has been cancelled. """

    id: str


class AnimationCreated(CDPEvent):
    """ Event for each animation that has been created. """

    id: str


class AnimationStarted(CDPEvent):
    """ Event for animation that has been started. """

    animation: Animation.Animation


class AnimationUpdated(CDPEvent):
    """ Event for animation that has been updated. """

    animation: Animation.Animation



from .events import (
    PrefetchStatusUpdated,
    PreloadEnabledStateUpdated,
    PreloadingAttemptSourcesUpdated,
    PrerenderStatusUpdated,
    RuleSetRemoved,
    RuleSetUpdated,
)
from .methods import Disable, Enable
from .types import (
    PrefetchStatus,
    PreloadingAttemptKey,
    PreloadingAttemptSource,
    PreloadingStatus,
    PreloadPipelineId,
    PrerenderFinalStatus,
    PrerenderMismatchedHeaders,
    RuleSet,
    RuleSetErrorType,
    RuleSetId,
    SpeculationAction,
    SpeculationTargetHint,
)

__all__ = [
    RuleSetId,
    RuleSet,
    RuleSetErrorType,
    SpeculationAction,
    SpeculationTargetHint,
    PreloadingAttemptKey,
    PreloadingAttemptSource,
    PreloadPipelineId,
    PrerenderFinalStatus,
    PreloadingStatus,
    PrefetchStatus,
    PrerenderMismatchedHeaders,
    RuleSetUpdated,
    RuleSetRemoved,
    PreloadEnabledStateUpdated,
    PrefetchStatusUpdated,
    PrerenderStatusUpdated,
    PreloadingAttemptSourcesUpdated,
    Enable,
    Disable,
]



from .events import (
    BackForwardCacheNotUsed,
    CompilationCacheProduced,
    DocumentOpened,
    DomContentEventFired,
    DownloadProgress,
    DownloadWillBegin,
    FileChooserOpened,
    FrameAttached,
    FrameClearedScheduledNavigation,
    FrameDetached,
    FrameNavigated,
    FrameRequestedNavigation,
    FrameResized,
    FrameScheduledNavigation,
    FrameStartedLoading,
    FrameStartedNavigating,
    FrameStoppedLoading,
    FrameSubtreeWillBeDetached,
    InterstitialHidden,
    InterstitialShown,
    JavascriptDialogClosed,
    JavascriptDialogOpening,
    LifecycleEvent,
    LoadEventFired,
    NavigatedWithinDocument,
    ScreencastFrame,
    ScreencastVisibilityChanged,
    WindowOpen,
)
from .methods import (
    AddCompilationCache,
    AddScriptToEvaluateOnLoad,
    AddScriptToEvaluateOnNewDocument,
    BringToFront,
    CaptureScreenshot,
    CaptureSnapshot,
    ClearCompilationCache,
    ClearDeviceMetricsOverride,
    ClearDeviceOrientationOverride,
    ClearGeolocationOverride,
    Close,
    Crash,
    CreateIsolatedWorld,
    DeleteCookie,
    Disable,
    Enable,
    GenerateTestReport,
    GetAdScriptAncestry,
    GetAnnotatedPageContent,
    GetAppId,
    GetAppManifest,
    GetFrameTree,
    GetInstallabilityErrors,
    GetLayoutMetrics,
    GetManifestIcons,
    GetNavigationHistory,
    GetOriginTrials,
    GetPermissionsPolicyState,
    GetResourceContent,
    GetResourceTree,
    HandleJavaScriptDialog,
    Navigate,
    NavigateToHistoryEntry,
    PrintToPDF,
    ProduceCompilationCache,
    Reload,
    RemoveScriptToEvaluateOnLoad,
    RemoveScriptToEvaluateOnNewDocument,
    ResetNavigationHistory,
    ScreencastFrameAck,
    SearchInResource,
    SetAdBlockingEnabled,
    SetBypassCSP,
    SetDeviceMetricsOverride,
    SetDeviceOrientationOverride,
    SetDocumentContent,
    SetDownloadBehavior,
    SetFontFamilies,
    SetFontSizes,
    SetGeolocationOverride,
    SetInterceptFileChooserDialog,
    SetLifecycleEventsEnabled,
    SetPrerenderingAllowed,
    SetRPHRegistrationMode,
    SetSPCTransactionMode,
    SetTouchEmulationEnabled,
    SetWebLifecycleState,
    StartScreencast,
    StopLoading,
    StopScreencast,
    WaitForDebugger,
)
from .types import (
    AdFrameExplanation,
    AdFrameStatus,
    AdFrameType,
    AdScriptAncestry,
    AdScriptId,
    AppManifestError,
    AppManifestParsedProperties,
    BackForwardCacheBlockingDetails,
    BackForwardCacheNotRestoredExplanation,
    BackForwardCacheNotRestoredExplanationTree,
    BackForwardCacheNotRestoredReason,
    BackForwardCacheNotRestoredReasonType,
    ClientNavigationDisposition,
    ClientNavigationReason,
    CompilationCacheParams,
    CrossOriginIsolatedContextType,
    DialogType,
    FileFilter,
    FileHandler,
    FontFamilies,
    FontSizes,
    Frame,
    FrameId,
    FrameResource,
    FrameResourceTree,
    FrameTree,
    GatedAPIFeatures,
    ImageResource,
    InstallabilityError,
    InstallabilityErrorArgument,
    LaunchHandler,
    LayoutViewport,
    NavigationEntry,
    NavigationType,
    OriginTrial,
    OriginTrialStatus,
    OriginTrialToken,
    OriginTrialTokenStatus,
    OriginTrialTokenWithStatus,
    OriginTrialUsageRestriction,
    PermissionsPolicyBlockLocator,
    PermissionsPolicyBlockReason,
    PermissionsPolicyFeature,
    PermissionsPolicyFeatureState,
    ProtocolHandler,
    ReferrerPolicy,
    RelatedApplication,
    ScopeExtension,
    ScreencastFrameMetadata,
    Screenshot,
    ScriptFontFamilies,
    ScriptIdentifier,
    SecureContextType,
    SecurityOriginDetails,
    ShareTarget,
    Shortcut,
    TransitionType,
    Viewport,
    VisualViewport,
    WebAppManifest,
)

__all__ = [
    FrameId,
    AdFrameType,
    AdFrameExplanation,
    AdFrameStatus,
    AdScriptId,
    AdScriptAncestry,
    SecureContextType,
    CrossOriginIsolatedContextType,
    GatedAPIFeatures,
    PermissionsPolicyFeature,
    PermissionsPolicyBlockReason,
    PermissionsPolicyBlockLocator,
    PermissionsPolicyFeatureState,
    OriginTrialTokenStatus,
    OriginTrialStatus,
    OriginTrialUsageRestriction,
    OriginTrialToken,
    OriginTrialTokenWithStatus,
    OriginTrial,
    SecurityOriginDetails,
    Frame,
    FrameResource,
    FrameResourceTree,
    FrameTree,
    ScriptIdentifier,
    TransitionType,
    NavigationEntry,
    ScreencastFrameMetadata,
    DialogType,
    AppManifestError,
    AppManifestParsedProperties,
    LayoutViewport,
    VisualViewport,
    Viewport,
    FontFamilies,
    ScriptFontFamilies,
    FontSizes,
    ClientNavigationReason,
    ClientNavigationDisposition,
    InstallabilityErrorArgument,
    InstallabilityError,
    ReferrerPolicy,
    CompilationCacheParams,
    FileFilter,
    FileHandler,
    ImageResource,
    LaunchHandler,
    ProtocolHandler,
    RelatedApplication,
    ScopeExtension,
    Screenshot,
    ShareTarget,
    Shortcut,
    WebAppManifest,
    NavigationType,
    BackForwardCacheNotRestoredReason,
    BackForwardCacheNotRestoredReasonType,
    BackForwardCacheBlockingDetails,
    BackForwardCacheNotRestoredExplanation,
    BackForwardCacheNotRestoredExplanationTree,
    DomContentEventFired,
    FileChooserOpened,
    FrameAttached,
    FrameClearedScheduledNavigation,
    FrameDetached,
    FrameSubtreeWillBeDetached,
    FrameNavigated,
    DocumentOpened,
    FrameResized,
    FrameStartedNavigating,
    FrameRequestedNavigation,
    FrameScheduledNavigation,
    FrameStartedLoading,
    FrameStoppedLoading,
    DownloadWillBegin,
    DownloadProgress,
    InterstitialHidden,
    InterstitialShown,
    JavascriptDialogClosed,
    JavascriptDialogOpening,
    LifecycleEvent,
    BackForwardCacheNotUsed,
    LoadEventFired,
    NavigatedWithinDocument,
    ScreencastFrame,
    ScreencastVisibilityChanged,
    WindowOpen,
    CompilationCacheProduced,
    AddScriptToEvaluateOnLoad,
    AddScriptToEvaluateOnNewDocument,
    BringToFront,
    CaptureScreenshot,
    CaptureSnapshot,
    ClearDeviceMetricsOverride,
    ClearDeviceOrientationOverride,
    ClearGeolocationOverride,
    CreateIsolatedWorld,
    DeleteCookie,
    Disable,
    Enable,
    GetAppManifest,
    GetInstallabilityErrors,
    GetManifestIcons,
    GetAppId,
    GetAdScriptAncestry,
    GetFrameTree,
    GetLayoutMetrics,
    GetNavigationHistory,
    ResetNavigationHistory,
    GetResourceContent,
    GetResourceTree,
    HandleJavaScriptDialog,
    Navigate,
    NavigateToHistoryEntry,
    PrintToPDF,
    Reload,
    RemoveScriptToEvaluateOnLoad,
    RemoveScriptToEvaluateOnNewDocument,
    ScreencastFrameAck,
    SearchInResource,
    SetAdBlockingEnabled,
    SetBypassCSP,
    GetPermissionsPolicyState,
    GetOriginTrials,
    SetDeviceMetricsOverride,
    SetDeviceOrientationOverride,
    SetFontFamilies,
    SetFontSizes,
    SetDocumentContent,
    SetDownloadBehavior,
    SetGeolocationOverride,
    SetLifecycleEventsEnabled,
    SetTouchEmulationEnabled,
    StartScreencast,
    StopLoading,
    Crash,
    Close,
    SetWebLifecycleState,
    StopScreencast,
    ProduceCompilationCache,
    AddCompilationCache,
    ClearCompilationCache,
    SetSPCTransactionMode,
    SetRPHRegistrationMode,
    GenerateTestReport,
    WaitForDebugger,
    SetInterceptFileChooserDialog,
    SetPrerenderingAllowed,
    GetAnnotatedPageContent,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Performance/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Performance,
)
from cdpkit.protocol.base import CDPEvent


class Metrics(CDPEvent):
    """ Current values of the metrics. """

    metrics: list[Performance.Metric]
    title: str



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Memory/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    MemoryDOMCounter,
    MemoryModule,
    MemoryPressureLevel,
    MemorySamplingProfile,
    MemorySamplingProfileNode,
)

PressureLevel = MemoryPressureLevel
SamplingProfileNode = MemorySamplingProfileNode
SamplingProfile = MemorySamplingProfile
Module = MemoryModule
DOMCounter = MemoryDOMCounter



from .browser import (
    ArgumentAlreadyExistsInOptions,
    BrowserLaunchError,
    ElementNotFileInput,
    ExecutableNotFoundError,
    NoSuchElement,
    NoValidTabError,
    ParamsMustSpecified,
    ScriptRunError,
    TabNotFoundError,
)
from .connection import (
    CallbackParameterError,
    CommandExecutionError,
    CommandExecutionTimeout,
    InvalidCallback,
    InvalidResponse,
    NetworkError,
    WebSocketConnectionClosed,
)
from .generate import GeneratorNameNotFound

__all__ = [
    'ExecutableNotFoundError',
    'GeneratorNameNotFound',
    'InvalidCallback',
    'NetworkError',
    'NoSuchElement',
    'TabNotFoundError',
    'CallbackParameterError',
    'InvalidResponse',
    'CommandExecutionTimeout',
    'WebSocketConnectionClosed',
    'BrowserLaunchError',
    'NoValidTabError',
    'ElementNotFileInput',
    'ArgumentAlreadyExistsInOptions',
    'ParamsMustSpecified',
    'ScriptRunError',
    'CommandExecutionError'
]



import asyncio
import inspect
import json
import re
from collections.abc import AsyncIterable, Callable
from contextlib import suppress
from typing import Any

import aiohttp
import websockets
from pydantic import BaseModel, PrivateAttr
from websockets.asyncio.client import ClientConnection
from websockets.protocol import State

from cdpkit.connection.manager import CommandsManager, EventsManager
from cdpkit.exception import (
    CallbackParameterError,
    CommandExecutionError,
    CommandExecutionTimeout,
    InvalidResponse,
    NetworkError,
    WebSocketConnectionClosed,
)
from cdpkit.logger import logger
from cdpkit.protocol import RESULT_TYPE, CDPEvent, CDPMethod, Target


class CDPSession(BaseModel):
    ws_endpoint: str
    target_id: Target.TargetID

    _receive_task: asyncio.Task | None = PrivateAttr(default=None)
    _ws_connection: ClientConnection | None = PrivateAttr(default=None)
    _commands_manager: CommandsManager = PrivateAttr(default=CommandsManager())
    _events_manager: EventsManager = PrivateAttr(default=EventsManager())

    async def _parse_ws_address(self) -> str:
        if self.target_id == 'browser':
            return await self.get_browser_ws_address()
        else:
            return f'ws://{self.ws_endpoint}/devtools/page/{self.target_id}'

    async def get_browser_ws_address(self) -> str:
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(f'http://{self.ws_endpoint}/json/version') as resp:
                    data = await resp.json()
                    return data['webSocketDebuggerUrl']
        except (aiohttp.ClientError, KeyError) as exc:
            raise NetworkError(f'Failed to get websocket address: {exc}')
        except KeyError as exc:
            raise InvalidResponse(f'Failed to get browser ws address: {exc}')

    async def _ensure_active_connection(self) -> None:
        if self._ws_connection is None or self._ws_connection.state is State.CLOSED:
            await self.establish_new_connection()

    async def establish_new_connection(self) -> None:
        ws_address = await self._parse_ws_address()
        logger.debug(f'ws_address: {ws_address}')

        self._ws_connection = await websockets.connect(
            ws_address,
            max_size=1024 * 1024 * 10  # 10MB
        )
        logger.info(f'start get page events: {ws_address}')
        self._receive_task = asyncio.create_task(self._receive_events())

    async def ping(self) -> bool:
        await self._ensure_active_connection()

        with suppress():
            await self._ws_connection.ping()
            return True
        return False

    async def execute(self, cdp_method: CDPMethod[RESULT_TYPE], timeout: int = 3) -> RESULT_TYPE:
        await self._ensure_active_connection()

        _id, future = self._commands_manager.create_command_future()
        command = cdp_method.command
        command['id'] = _id
        logger.info(f'execute command: {command}')

        try:
            await self._ws_connection.send(json.dumps(command))
            response: str = await asyncio.wait_for(future, timeout)
            resp_json = json.loads(response)
            if 'error' in resp_json:
                raise CommandExecutionError(f'Command {command} execution failed: {resp_json["error"]}')
            return await cdp_method.parse_response(json.dumps(resp_json['result']))
        except TimeoutError:
            self._commands_manager.remove_pending_command(_id)
            raise CommandExecutionTimeout()
        except websockets.ConnectionClosed:
            await self.close()
            raise WebSocketConnectionClosed()

    async def close(self) -> None:
        await self.clear_callbacks()

        if self._ws_connection:
            with suppress(websockets.ConnectionClosed):
                await self._ws_connection.close()

        self._ws_connection = None

        if self._receive_task and not self._receive_task.done():
            self._receive_task.cancel()

        logger.info('Connection resources cleaned up')

    async def _incoming_messages(self) -> AsyncIterable[websockets.Data]:
        while self._ws_connection.state is not State.CLOSED:
            yield await self._ws_connection.recv()

    async def _receive_events(self) -> None:
        try:
            async for raw_message in self._incoming_messages():
                await self._process_single_message(raw_message)
        except websockets.ConnectionClosed as exc:
            logger.info(f'Connection closed gracefully: {exc}')
        except Exception as exc:
            logger.error(f'Unexpected error in event loop: {exc}')
            raise exc

    async def _process_single_message(self, raw_message: str) -> None:
        message = await self._parse_message(raw_message)
        if message is None:
            return

        if await self._is_command_response(message):
            await self._handle_command_message(message)
        else:
            await self._handle_event_message(message)

    @staticmethod
    async def _is_command_response(message: dict[str, Any]) -> bool:
        return isinstance(message.get('id'), int)

    @staticmethod
    async def _parse_message(raw_message: str) -> dict[str, Any] | None:
        try:
            return json.loads(raw_message)
        except json.JSONDecodeError as exc:
            logger.warning(f'Failed to parse raw message: {raw_message[:200]}, {exc}')
            return None

    async def _handle_command_message(self, message: dict[str, Any]) -> None:
        logger.info(f'Processing command response: {message["id"]}')

        self._commands_manager.resolve_command(message)

    async def _handle_event_message(self, message: dict[str, Any]) -> None:
        logger.info(f'Processing event message: {message}')

        if 'method' in message:
            await self._events_manager.process_event(message)
        else:
            logger.warning('unknown event')

    def __str__(self) -> str:
        return f'CDPSession(ws_endpoint={self.ws_endpoint}, target_id={self.target_id})'

    def __repr__(self) -> str:
        return self.__str__()

    async def register_callback(self, event: type[CDPEvent], callback: Callable, temporary: bool = False) -> int:
        return await self._events_manager.register_callback(
            event=event,
            callback=callback,
            temporary=temporary
        )

    async def remove_callback(self, callback_id: int) -> bool:
        return await self._events_manager.remove_callback(callback_id)

    async def clear_callbacks(self):
        await self._events_manager.clear_callbacks()


class CDPSessionManager(BaseModel):
    ws_endpoint: str

    _connection_session: dict[str, CDPSession] = PrivateAttr(default_factory=dict)

    async def remove_session(self, target_id: Target.TargetID = 'browser') -> None:
        if target_id in self._connection_session:
            await self._connection_session[target_id].close()
            del self._connection_session[target_id]

    async def get_session(self, target_id: Target.TargetID = 'browser') -> CDPSession:
        if target_id not in self._connection_session:
            cdp_session = CDPSession(
                ws_endpoint=self.ws_endpoint,
                target_id=target_id,
            )
            self._connection_session[target_id] = cdp_session
        else:
            cdp_session = self._connection_session[target_id]

        return cdp_session

    def __str__(self):
        return f'CDPSessionManager(ws_endpoint={self.ws_endpoint})'

    def __repr__(self) -> str:
        return str(self)


class CDPSessionExecutor(BaseModel):
    session: CDPSession | None = None
    session_manager: CDPSessionManager | None = None

    async def on(self, event: type[CDPEvent], callback: callable, temporary: bool = False) -> int:
        """

        Examples:
            async def _on_target_created(event_data: Target.TargetCreated):
                ...

            await session.on(event=TargetCreated, callback=_on_target_created)
        """
        sig = inspect.signature(callback)
        if 'event_data' in sig.parameters:
            # raise CallbackParameterError('Required parameter "event_data" not found in callback function')
            event_data_type = sig.parameters["event_data"].annotation

            event_name = re.sub(
                    r'(?<=\.)([a-z])',
                    lambda m: m.group(1).upper(),
                    event.EVENT_NAME
                )
            if isinstance(event_data_type, str):
                if not (event_data_type == event_name):
                    raise CallbackParameterError(
                        f"Parameter 'event_data' type mismatch. "
                        f"Expected {event_name}, but got {event_data_type}."
                    )
            elif not issubclass(event_data_type, event):
                raise CallbackParameterError(
                    f"Parameter 'event_data' type mismatch. "
                    f"Expected {event_name}, but got {event_data_type.__name__}."
                )
        return await self.session.register_callback(
            event, callback, temporary
        )

    async def execute_method(self, cdp_method: CDPMethod[RESULT_TYPE], timeout: int = 60) -> RESULT_TYPE:
        return await self.session.execute(
            cdp_method,
            timeout
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/DeviceAccess/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DeviceAccess,
)
from cdpkit.protocol.base import CDPEvent


class DeviceRequestPrompted(CDPEvent):
    """ A device request opened a user prompt to select a device. Respond with the
    selectPrompt or cancelPrompt command. """

    id: DeviceAccess.RequestId
    devices: list[DeviceAccess.PromptDevice]



class CustomException(Exception):
    """
    Base custom exception class

    The base class for all custom exceptions, supporting default or custom error messages
    """
    ERROR_INFO = ''

    def __init__(self, error_info: str | None = None):
        """
        Initialize the exception instance

        Args:
            error_info (str | None, optional):
                Custom error message. If None, use the default value from the class attribute ERROR_INFO
                Default: None
        """
        if error_info is None:
            error_info = self.ERROR_INFO
        super().__init__(error_info)



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/ServiceWorker/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    ServiceWorker,
)
from cdpkit.protocol.base import CDPMethod, InputModel


class DeliverPushMessageInput(InputModel):

    origin: str
    registrationId: ServiceWorker.RegistrationID
    data: str


class DeliverPushMessage(CDPMethod[None]):

    INPUT_VALIDATOR = DeliverPushMessageInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        origin: str,
        registration_id: ServiceWorker.RegistrationID,
        data: str
    ):
        super().__init__(
            origin=origin,
            registrationId=registration_id,
            data=data
        )


class Disable(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class DispatchSyncEventInput(InputModel):

    origin: str
    registrationId: ServiceWorker.RegistrationID
    tag: str
    lastChance: bool


class DispatchSyncEvent(CDPMethod[None]):

    INPUT_VALIDATOR = DispatchSyncEventInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        origin: str,
        registration_id: ServiceWorker.RegistrationID,
        tag: str,
        last_chance: bool
    ):
        super().__init__(
            origin=origin,
            registrationId=registration_id,
            tag=tag,
            lastChance=last_chance
        )


class DispatchPeriodicSyncEventInput(InputModel):

    origin: str
    registrationId: ServiceWorker.RegistrationID
    tag: str


class DispatchPeriodicSyncEvent(CDPMethod[None]):

    INPUT_VALIDATOR = DispatchPeriodicSyncEventInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        origin: str,
        registration_id: ServiceWorker.RegistrationID,
        tag: str
    ):
        super().__init__(
            origin=origin,
            registrationId=registration_id,
            tag=tag
        )


class Enable(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class SetForceUpdateOnPageLoadInput(InputModel):

    forceUpdateOnPageLoad: bool


class SetForceUpdateOnPageLoad(CDPMethod[None]):

    INPUT_VALIDATOR = SetForceUpdateOnPageLoadInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        force_update_on_page_load: bool
    ):
        super().__init__(
            forceUpdateOnPageLoad=force_update_on_page_load
        )


class SkipWaitingInput(InputModel):

    scopeURL: str


class SkipWaiting(CDPMethod[None]):

    INPUT_VALIDATOR = SkipWaitingInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        scope_url: str
    ):
        super().__init__(
            scopeURL=scope_url
        )


class StartWorkerInput(InputModel):

    scopeURL: str


class StartWorker(CDPMethod[None]):

    INPUT_VALIDATOR = StartWorkerInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        scope_url: str
    ):
        super().__init__(
            scopeURL=scope_url
        )


class StopAllWorkers(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class StopWorkerInput(InputModel):

    versionId: str


class StopWorker(CDPMethod[None]):

    INPUT_VALIDATOR = StopWorkerInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        version_id: str
    ):
        super().__init__(
            versionId=version_id
        )


class UnregisterInput(InputModel):

    scopeURL: str


class Unregister(CDPMethod[None]):

    INPUT_VALIDATOR = UnregisterInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        scope_url: str
    ):
        super().__init__(
            scopeURL=scope_url
        )


class UpdateRegistrationInput(InputModel):

    scopeURL: str


class UpdateRegistration(CDPMethod[None]):

    INPUT_VALIDATOR = UpdateRegistrationInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        scope_url: str
    ):
        super().__init__(
            scopeURL=scope_url
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows configuring virtual Bluetooth devices to test
the web-bluetooth API.
https://chromedevtools.github.io/devtools-protocol/tot/BluetoothEmulation/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    BluetoothEmulation,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class EnableInput(InputModel):

    state: BluetoothEmulation.CentralState
    leSupported: bool


class Enable(CDPMethod[None]):
    """ Enable the BluetoothEmulation domain. """

    INPUT_VALIDATOR = EnableInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        state: BluetoothEmulation.CentralState,
        le_supported: bool
    ):
        super().__init__(
            state=state,
            leSupported=le_supported
        )


class SetSimulatedCentralStateInput(InputModel):

    state: BluetoothEmulation.CentralState


class SetSimulatedCentralState(CDPMethod[None]):
    """ Set the state of the simulated central. """

    INPUT_VALIDATOR = SetSimulatedCentralStateInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        state: BluetoothEmulation.CentralState
    ):
        super().__init__(
            state=state
        )


class Disable(CDPMethod[None]):
    """ Disable the BluetoothEmulation domain. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class SimulatePreconnectedPeripheralInput(InputModel):

    address: str
    name: str
    manufacturerData: list[BluetoothEmulation.ManufacturerData]
    knownServiceUuids: list[str]


class SimulatePreconnectedPeripheral(CDPMethod[None]):
    """ Simulates a peripheral with |address|, |name| and |knownServiceUuids|
    that has already been connected to the system. """

    INPUT_VALIDATOR = SimulatePreconnectedPeripheralInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        address: str,
        name: str,
        manufacturer_data: list[BluetoothEmulation.ManufacturerData],
        known_service_uuids: list[str]
    ):
        super().__init__(
            address=address,
            name=name,
            manufacturerData=manufacturer_data,
            knownServiceUuids=known_service_uuids
        )


class SimulateAdvertisementInput(InputModel):

    entry: BluetoothEmulation.ScanEntry


class SimulateAdvertisement(CDPMethod[None]):
    """ Simulates an advertisement packet described in |entry| being received by
    the central. """

    INPUT_VALIDATOR = SimulateAdvertisementInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        entry: BluetoothEmulation.ScanEntry
    ):
        super().__init__(
            entry=entry
        )


class SimulateGATTOperationResponseInput(InputModel):

    address: str
    type: BluetoothEmulation.GATTOperationType
    code: int


class SimulateGATTOperationResponse(CDPMethod[None]):
    """ Simulates the response code from the peripheral with |address| for a
    GATT operation of |type|. The |code| value follows the HCI Error Codes from
    Bluetooth Core Specification Vol 2 Part D 1.3 List Of Error Codes. """

    INPUT_VALIDATOR = SimulateGATTOperationResponseInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        address: str,
        type_: BluetoothEmulation.GATTOperationType,
        code: int
    ):
        super().__init__(
            address=address,
            type=type_,
            code=code
        )


class SimulateCharacteristicOperationResponseInput(InputModel):

    characteristicId: str
    type: BluetoothEmulation.CharacteristicOperationType
    code: int
    data: str | None = None


class SimulateCharacteristicOperationResponse(CDPMethod[None]):
    """ Simulates the response from the characteristic with |characteristicId| for a
    characteristic operation of |type|. The |code| value follows the Error
    Codes from Bluetooth Core Specification Vol 3 Part F 3.4.1.1 Error Response.
    The |data| is expected to exist when simulating a successful read operation
    response. """

    INPUT_VALIDATOR = SimulateCharacteristicOperationResponseInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        characteristic_id: str,
        type_: BluetoothEmulation.CharacteristicOperationType,
        code: int,
        data: str | None = None
    ):
        super().__init__(
            characteristicId=characteristic_id,
            type=type_,
            code=code,
            data=data
        )


class SimulateDescriptorOperationResponseInput(InputModel):

    descriptorId: str
    type: BluetoothEmulation.DescriptorOperationType
    code: int
    data: str | None = None


class SimulateDescriptorOperationResponse(CDPMethod[None]):
    """ Simulates the response from the descriptor with |descriptorId| for a
    descriptor operation of |type|. The |code| value follows the Error
    Codes from Bluetooth Core Specification Vol 3 Part F 3.4.1.1 Error Response.
    The |data| is expected to exist when simulating a successful read operation
    response. """

    INPUT_VALIDATOR = SimulateDescriptorOperationResponseInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        descriptor_id: str,
        type_: BluetoothEmulation.DescriptorOperationType,
        code: int,
        data: str | None = None
    ):
        super().__init__(
            descriptorId=descriptor_id,
            type=type_,
            code=code,
            data=data
        )


class AddServiceInput(InputModel):

    address: str
    serviceUuid: str


class AddServiceOutput(OutputModel):

    serviceId: str


class AddService(CDPMethod[AddServiceOutput]):
    """ Adds a service with |serviceUuid| to the peripheral with |address|. """

    INPUT_VALIDATOR = AddServiceInput
    OUTPUT_VALIDATOR = AddServiceOutput

    def __init__(
        self,
        *,
        address: str,
        service_uuid: str
    ):
        super().__init__(
            address=address,
            serviceUuid=service_uuid
        )


class RemoveServiceInput(InputModel):

    serviceId: str


class RemoveService(CDPMethod[None]):
    """ Removes the service respresented by |serviceId| from the simulated central. """

    INPUT_VALIDATOR = RemoveServiceInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        service_id: str
    ):
        super().__init__(
            serviceId=service_id
        )


class AddCharacteristicInput(InputModel):

    serviceId: str
    characteristicUuid: str
    properties: BluetoothEmulation.CharacteristicProperties


class AddCharacteristicOutput(OutputModel):

    characteristicId: str


class AddCharacteristic(CDPMethod[AddCharacteristicOutput]):
    """ Adds a characteristic with |characteristicUuid| and |properties| to the
    service represented by |serviceId|. """

    INPUT_VALIDATOR = AddCharacteristicInput
    OUTPUT_VALIDATOR = AddCharacteristicOutput

    def __init__(
        self,
        *,
        service_id: str,
        characteristic_uuid: str,
        properties: BluetoothEmulation.CharacteristicProperties
    ):
        super().__init__(
            serviceId=service_id,
            characteristicUuid=characteristic_uuid,
            properties=properties
        )


class RemoveCharacteristicInput(InputModel):

    characteristicId: str


class RemoveCharacteristic(CDPMethod[None]):
    """ Removes the characteristic respresented by |characteristicId| from the
    simulated central. """

    INPUT_VALIDATOR = RemoveCharacteristicInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        characteristic_id: str
    ):
        super().__init__(
            characteristicId=characteristic_id
        )


class AddDescriptorInput(InputModel):

    characteristicId: str
    descriptorUuid: str


class AddDescriptorOutput(OutputModel):

    descriptorId: str


class AddDescriptor(CDPMethod[AddDescriptorOutput]):
    """ Adds a descriptor with |descriptorUuid| to the characteristic respresented
    by |characteristicId|. """

    INPUT_VALIDATOR = AddDescriptorInput
    OUTPUT_VALIDATOR = AddDescriptorOutput

    def __init__(
        self,
        *,
        characteristic_id: str,
        descriptor_uuid: str
    ):
        super().__init__(
            characteristicId=characteristic_id,
            descriptorUuid=descriptor_uuid
        )


class RemoveDescriptorInput(InputModel):

    descriptorId: str


class RemoveDescriptor(CDPMethod[None]):
    """ Removes the descriptor with |descriptorId| from the simulated central. """

    INPUT_VALIDATOR = RemoveDescriptorInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        descriptor_id: str
    ):
        super().__init__(
            descriptorId=descriptor_id
        )


class SimulateGATTDisconnectionInput(InputModel):

    address: str


class SimulateGATTDisconnection(CDPMethod[None]):
    """ Simulates a GATT disconnection from the peripheral with |address|. """

    INPUT_VALIDATOR = SimulateGATTDisconnectionInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        address: str
    ):
        super().__init__(
            address=address
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Profiler/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Profiler,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class Disable(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetBestEffortCoverageOutput(OutputModel):

    result: list[Profiler.ScriptCoverage]


class GetBestEffortCoverage(CDPMethod[GetBestEffortCoverageOutput]):
    """ Collect coverage data for the current isolate. The coverage data may be incomplete due to
    garbage collection. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetBestEffortCoverageOutput


class SetSamplingIntervalInput(InputModel):

    interval: int


class SetSamplingInterval(CDPMethod[None]):
    """ Changes CPU profiler sampling interval. Must be called before CPU profiles recording started. """

    INPUT_VALIDATOR = SetSamplingIntervalInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        interval: int
    ):
        super().__init__(
            interval=interval
        )


class Start(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class StartPreciseCoverageInput(InputModel):

    callCount: bool | None = None
    detailed: bool | None = None
    allowTriggeredUpdates: bool | None = None


class StartPreciseCoverageOutput(OutputModel):

    timestamp: float


class StartPreciseCoverage(CDPMethod[StartPreciseCoverageOutput]):
    """ Enable precise code coverage. Coverage data for JavaScript executed before enabling precise code
    coverage may be incomplete. Enabling prevents running optimized code and resets execution
    counters. """

    INPUT_VALIDATOR = StartPreciseCoverageInput
    OUTPUT_VALIDATOR = StartPreciseCoverageOutput

    def __init__(
        self,
        *,
        call_count: bool | None = None,
        detailed: bool | None = None,
        allow_triggered_updates: bool | None = None
    ):
        super().__init__(
            callCount=call_count,
            detailed=detailed,
            allowTriggeredUpdates=allow_triggered_updates
        )


class StopOutput(OutputModel):

    profile: Profiler.Profile


class Stop(CDPMethod[StopOutput]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = StopOutput


class StopPreciseCoverage(CDPMethod[None]):
    """ Disable precise code coverage. Disabling releases unnecessary execution count records and allows
    executing optimized code. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class TakePreciseCoverageOutput(OutputModel):

    result: list[Profiler.ScriptCoverage]
    timestamp: float


class TakePreciseCoverage(CDPMethod[TakePreciseCoverageOutput]):
    """ Collect coverage data for the current isolate, and resets execution counters. Precise code
    coverage needs to have started. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = TakePreciseCoverageOutput



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Defines commands and events for Autofill.
https://chromedevtools.github.io/devtools-protocol/tot/Autofill/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    AutofillAddress,
    AutofillAddressField,
    AutofillAddressFields,
    AutofillAddressUI,
    AutofillCreditCard,
    AutofillFilledField,
    AutofillFillingStrategy,
)

CreditCard = AutofillCreditCard
AddressField = AutofillAddressField
AddressFields = AutofillAddressFields
Address = AutofillAddress
AddressUI = AutofillAddressUI
FillingStrategy = AutofillFillingStrategy
FilledField = AutofillFilledField



from .methods import DeleteCache, DeleteEntry, RequestCachedResponse, RequestCacheNames, RequestEntries
from .types import Cache, CachedResponse, CachedResponseType, CacheId, DataEntry, Header

__all__ = [
    CacheId,
    CachedResponseType,
    DataEntry,
    Cache,
    Header,
    CachedResponse,
    DeleteCache,
    DeleteEntry,
    RequestCacheNames,
    RequestCachedResponse,
    RequestEntries,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Inspector/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol.base import CDPMethod


class Disable(CDPMethod[None]):
    """ Disables inspector domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables inspector domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/IndexedDB/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    IndexedDBDatabaseWithObjectStores,
    IndexedDBDataEntry,
    IndexedDBKey,
    IndexedDBKeyPath,
    IndexedDBKeyRange,
    IndexedDBObjectStore,
    IndexedDBObjectStoreIndex,
)

DatabaseWithObjectStores = IndexedDBDatabaseWithObjectStores
ObjectStore = IndexedDBObjectStore
ObjectStoreIndex = IndexedDBObjectStoreIndex
Key = IndexedDBKey
KeyRange = IndexedDBKeyRange
DataEntry = IndexedDBDataEntry
KeyPath = IndexedDBKeyPath



import sys
from enum import IntEnum

from loguru import logger as _loguru_logger

__all__ = [
    'logger'
]


class LogLevel(IntEnum):
    """Log level enumeration class, inherits from IntEnum"""
    TRACE = 5
    DEBUG = 10
    INFO = 20
    SUCCESS = 25
    WARNING = 30
    ERROR = 40
    CRITICAL = 50


def set_logger(
    print_level: str | LogLevel = LogLevel.SUCCESS,
):
    """Configure and return a logger instance

    Args:
        print_level (str | LogLevel, optional):
            Log level to use. Defaults to LogLevel.INFO.

    Returns:
        loguru._logger.Logger: Configured logger instance.

    """
    _format = '{time:YYYY-MM-DD HH:mm:ss.SSS} |<lvl>{level:8}</>| {name:8} : {module}:{line:4} | - <lvl>{message}</>'

    if isinstance(print_level, str):
        try:
            print_level = LogLevel[print_level]
        except KeyError:
            raise KeyError(f'Invalid log level {print_level}')

    _loguru_logger.configure(
        handlers=[
            {
                'sink': sys.stdout,
                'format': _format,
                'colorize': True,
                'level': print_level.value,
                'enqueue': True
            }
        ]
    )
    return _loguru_logger


logger = set_logger(LogLevel.DEBUG)



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows configuring virtual authenticators to test the WebAuthn
API.
https://chromedevtools.github.io/devtools-protocol/tot/WebAuthn/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    WebAuthn,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class EnableInput(InputModel):

    enableUI: bool | None = None


class Enable(CDPMethod[None]):
    """ Enable the WebAuthn domain and start intercepting credential storage and
    retrieval with a virtual authenticator. """

    INPUT_VALIDATOR = EnableInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        enable_ui: bool | None = None
    ):
        super().__init__(
            enableUI=enable_ui
        )


class Disable(CDPMethod[None]):
    """ Disable the WebAuthn domain. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class AddVirtualAuthenticatorInput(InputModel):

    options: WebAuthn.VirtualAuthenticatorOptions


class AddVirtualAuthenticatorOutput(OutputModel):

    authenticatorId: WebAuthn.AuthenticatorId


class AddVirtualAuthenticator(CDPMethod[AddVirtualAuthenticatorOutput]):
    """ Creates and adds a virtual authenticator. """

    INPUT_VALIDATOR = AddVirtualAuthenticatorInput
    OUTPUT_VALIDATOR = AddVirtualAuthenticatorOutput

    def __init__(
        self,
        *,
        options: WebAuthn.VirtualAuthenticatorOptions
    ):
        super().__init__(
            options=options
        )


class SetResponseOverrideBitsInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    isBogusSignature: bool | None = None
    isBadUV: bool | None = None
    isBadUP: bool | None = None


class SetResponseOverrideBits(CDPMethod[None]):
    """ Resets parameters isBogusSignature, isBadUV, isBadUP to false if they are not present. """

    INPUT_VALIDATOR = SetResponseOverrideBitsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        is_bogus_signature: bool | None = None,
        is_bad_uv: bool | None = None,
        is_bad_up: bool | None = None
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            isBogusSignature=is_bogus_signature,
            isBadUV=is_bad_uv,
            isBadUP=is_bad_up
        )


class RemoveVirtualAuthenticatorInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId


class RemoveVirtualAuthenticator(CDPMethod[None]):
    """ Removes the given authenticator. """

    INPUT_VALIDATOR = RemoveVirtualAuthenticatorInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId
    ):
        super().__init__(
            authenticatorId=authenticator_id
        )


class AddCredentialInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    credential: WebAuthn.Credential


class AddCredential(CDPMethod[None]):
    """ Adds the credential to the specified authenticator. """

    INPUT_VALIDATOR = AddCredentialInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        credential: WebAuthn.Credential
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            credential=credential
        )


class GetCredentialInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    credentialId: str


class GetCredentialOutput(OutputModel):

    credential: WebAuthn.Credential


class GetCredential(CDPMethod[GetCredentialOutput]):
    """ Returns a single credential stored in the given virtual authenticator that
    matches the credential ID. """

    INPUT_VALIDATOR = GetCredentialInput
    OUTPUT_VALIDATOR = GetCredentialOutput

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        credential_id: str
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            credentialId=credential_id
        )


class GetCredentialsInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId


class GetCredentialsOutput(OutputModel):

    credentials: list[WebAuthn.Credential]


class GetCredentials(CDPMethod[GetCredentialsOutput]):
    """ Returns all the credentials stored in the given virtual authenticator. """

    INPUT_VALIDATOR = GetCredentialsInput
    OUTPUT_VALIDATOR = GetCredentialsOutput

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId
    ):
        super().__init__(
            authenticatorId=authenticator_id
        )


class RemoveCredentialInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    credentialId: str


class RemoveCredential(CDPMethod[None]):
    """ Removes a credential from the authenticator. """

    INPUT_VALIDATOR = RemoveCredentialInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        credential_id: str
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            credentialId=credential_id
        )


class ClearCredentialsInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId


class ClearCredentials(CDPMethod[None]):
    """ Clears all the credentials from the specified device. """

    INPUT_VALIDATOR = ClearCredentialsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId
    ):
        super().__init__(
            authenticatorId=authenticator_id
        )


class SetUserVerifiedInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    isUserVerified: bool


class SetUserVerified(CDPMethod[None]):
    """ Sets whether User Verification succeeds or fails for an authenticator.
    The default is true. """

    INPUT_VALIDATOR = SetUserVerifiedInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        is_user_verified: bool
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            isUserVerified=is_user_verified
        )


class SetAutomaticPresenceSimulationInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    enabled: bool


class SetAutomaticPresenceSimulation(CDPMethod[None]):
    """ Sets whether tests of user presence will succeed immediately (if true) or fail to resolve (if false) for an authenticator.
    The default is true. """

    INPUT_VALIDATOR = SetAutomaticPresenceSimulationInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        enabled: bool
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            enabled=enabled
        )


class SetCredentialPropertiesInput(InputModel):

    authenticatorId: WebAuthn.AuthenticatorId
    credentialId: str
    backupEligibility: bool | None = None
    backupState: bool | None = None


class SetCredentialProperties(CDPMethod[None]):
    """ Allows setting credential properties.
    https://w3c.github.io/webauthn/#sctn-automation-set-credential-properties """

    INPUT_VALIDATOR = SetCredentialPropertiesInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        authenticator_id: WebAuthn.AuthenticatorId,
        credential_id: str,
        backup_eligibility: bool | None = None,
        backup_state: bool | None = None
    ):
        super().__init__(
            authenticatorId=authenticator_id,
            credentialId=credential_id,
            backupEligibility=backup_eligibility,
            backupState=backup_state
        )



import asyncio
import inspect
from collections import defaultdict
from collections.abc import Callable
from functools import partial

from pydantic import BaseModel, PrivateAttr

from cdpkit.exception import InvalidCallback
from cdpkit.logger import logger
from cdpkit.protocol import CDPEvent


class EventsManager(BaseModel):
    _pending_events: dict[int, dict] = PrivateAttr(default_factory=dict)
    _callback_id: int = PrivateAttr(default=0)

    _events_callbacks: dict[str, list[int]] = PrivateAttr(default=defaultdict(list))

    async def register_callback(
        self, event: type[CDPEvent], callback: Callable, temporary: bool = False
    ) -> int:
        if not callable(callback):
            logger.error('Callback must be callable function.')
            raise InvalidCallback()

        self._callback_id += 1
        self._pending_events[self._callback_id] = {
            'event': event.EVENT_NAME,
            'callback': callback,
            'callback_event': event,
            'temporary': temporary
        }
        self._events_callbacks[event.EVENT_NAME].append(self._callback_id)

        return self._callback_id

    async def remove_callback(self, callback_id: int) -> bool:
        if callback_id not in self._pending_events:
            logger.warning(f'No pending message can be resolved for id {callback_id}')
            return False

        callback_info = self._pending_events[callback_id]
        self._events_callbacks[callback_info['event']].remove(callback_id)
        del self._pending_events[callback_id]

        return True

    async def clear_callbacks(self):
        self._pending_events.clear()
        self._events_callbacks.clear()

    async def process_event(self, event_data: dict):
        event_name = event_data.get('method')
        logger.info(f'Processing event: {event_name}')

        await self._trigger_callbacks(event_name, event_data)

    async def _trigger_callbacks(self, event_name: str, event_data: dict):
        if event_name not in self._events_callbacks:
            return

        callbacks_to_remove = []

        for callback_id in self._events_callbacks[event_name]:
            callback_info = self._pending_events[callback_id]
            if callback_info['temporary']:
                callbacks_to_remove.append(callback_id)

            callback_func = callback_info['callback']

            sig = inspect.signature(callback_func)
            if 'event_data' in sig.parameters:
                event_data = callback_info['callback_event'].model_validate(event_data['params'])
                callback_func = partial(callback_func, event_data=event_data)

            try:
                if asyncio.iscoroutinefunction(callback_func):
                    await callback_func()
                else:
                    callback_func()
            except Exception as exc:
                logger.error(f'Error processing callback {event_name} {event_data}: {exc}')

        for callback_id in callbacks_to_remove:
            await self.remove_callback(callback_id)



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows interacting with the FedCM dialog.
https://chromedevtools.github.io/devtools-protocol/tot/FedCm/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    FedCm,
)
from cdpkit.protocol.base import CDPMethod, InputModel


class EnableInput(InputModel):

    disableRejectionDelay: bool | None = None


class Enable(CDPMethod[None]):

    INPUT_VALIDATOR = EnableInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        disable_rejection_delay: bool | None = None
    ):
        super().__init__(
            disableRejectionDelay=disable_rejection_delay
        )


class Disable(CDPMethod[None]):

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class SelectAccountInput(InputModel):

    dialogId: str
    accountIndex: int


class SelectAccount(CDPMethod[None]):

    INPUT_VALIDATOR = SelectAccountInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        dialog_id: str,
        account_index: int
    ):
        super().__init__(
            dialogId=dialog_id,
            accountIndex=account_index
        )


class ClickDialogButtonInput(InputModel):

    dialogId: str
    dialogButton: FedCm.DialogButton


class ClickDialogButton(CDPMethod[None]):

    INPUT_VALIDATOR = ClickDialogButtonInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        dialog_id: str,
        dialog_button: FedCm.DialogButton
    ):
        super().__init__(
            dialogId=dialog_id,
            dialogButton=dialog_button
        )


class OpenUrlInput(InputModel):

    dialogId: str
    accountIndex: int
    accountUrlType: FedCm.AccountUrlType


class OpenUrl(CDPMethod[None]):

    INPUT_VALIDATOR = OpenUrlInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        dialog_id: str,
        account_index: int,
        account_url_type: FedCm.AccountUrlType
    ):
        super().__init__(
            dialogId=dialog_id,
            accountIndex=account_index,
            accountUrlType=account_url_type
        )


class DismissDialogInput(InputModel):

    dialogId: str
    triggerCooldown: bool | None = None


class DismissDialog(CDPMethod[None]):

    INPUT_VALIDATOR = DismissDialogInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        dialog_id: str,
        trigger_cooldown: bool | None = None
    ):
        super().__init__(
            dialogId=dialog_id,
            triggerCooldown=trigger_cooldown
        )


class ResetCooldown(CDPMethod[None]):
    """ Resets the cooldown time, if any, to allow the next FedCM call to show
    a dialog even if one was recently dismissed by the user. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



from .events import (
    DataReceived,
    DirectTCPSocketAborted,
    DirectTCPSocketChunkReceived,
    DirectTCPSocketChunkSent,
    DirectTCPSocketClosed,
    DirectTCPSocketCreated,
    DirectTCPSocketOpened,
    DirectUDPSocketAborted,
    DirectUDPSocketChunkReceived,
    DirectUDPSocketChunkSent,
    DirectUDPSocketClosed,
    DirectUDPSocketCreated,
    DirectUDPSocketJoinedMulticastGroup,
    DirectUDPSocketLeftMulticastGroup,
    DirectUDPSocketOpened,
    EventSourceMessageReceived,
    LoadingFailed,
    LoadingFinished,
    PolicyUpdated,
    ReportingApiEndpointsChangedForOrigin,
    ReportingApiReportAdded,
    ReportingApiReportUpdated,
    RequestIntercepted,
    RequestServedFromCache,
    RequestWillBeSent,
    RequestWillBeSentExtraInfo,
    ResourceChangedPriority,
    ResponseReceived,
    ResponseReceivedEarlyHints,
    ResponseReceivedExtraInfo,
    SignedExchangeReceived,
    TrustTokenOperationDone,
    WebSocketClosed,
    WebSocketCreated,
    WebSocketFrameError,
    WebSocketFrameReceived,
    WebSocketFrameSent,
    WebSocketHandshakeResponseReceived,
    WebSocketWillSendHandshakeRequest,
    WebTransportClosed,
    WebTransportConnectionEstablished,
    WebTransportCreated,
)
from .methods import (
    CanClearBrowserCache,
    CanClearBrowserCookies,
    CanEmulateNetworkConditions,
    ClearAcceptedEncodingsOverride,
    ClearBrowserCache,
    ClearBrowserCookies,
    ConfigureDurableMessages,
    ContinueInterceptedRequest,
    DeleteCookies,
    Disable,
    EmulateNetworkConditions,
    EmulateNetworkConditionsByRule,
    Enable,
    EnableReportingApi,
    GetAllCookies,
    GetCertificate,
    GetCookies,
    GetRequestPostData,
    GetResponseBody,
    GetResponseBodyForInterception,
    GetSecurityIsolationStatus,
    LoadNetworkResource,
    OverrideNetworkState,
    ReplayXHR,
    SearchInResponseBody,
    SetAcceptedEncodings,
    SetAttachDebugStack,
    SetBlockedURLs,
    SetBypassServiceWorker,
    SetCacheDisabled,
    SetCookie,
    SetCookieControls,
    SetCookies,
    SetExtraHTTPHeaders,
    SetRequestInterception,
    SetUserAgentOverride,
    StreamResourceContent,
    TakeResponseBodyForInterceptionAsStream,
)
from .types import (
    AlternateProtocolUsage,
    AssociatedCookie,
    AuthChallenge,
    AuthChallengeResponse,
    BlockedReason,
    BlockedSetCookieWithReason,
    BlockPattern,
    CachedResource,
    CertificateTransparencyCompliance,
    ClientSecurityState,
    ConnectionType,
    ConnectTiming,
    ContentEncoding,
    ContentSecurityPolicySource,
    ContentSecurityPolicyStatus,
    Cookie,
    CookieBlockedReason,
    CookieExemptionReason,
    CookieParam,
    CookiePartitionKey,
    CookiePriority,
    CookieSameSite,
    CookieSourceScheme,
    CorsError,
    CorsErrorStatus,
    CrossOriginEmbedderPolicyStatus,
    CrossOriginEmbedderPolicyValue,
    CrossOriginOpenerPolicyStatus,
    CrossOriginOpenerPolicyValue,
    DirectSocketDnsQueryType,
    DirectTCPSocketOptions,
    DirectUDPMessage,
    DirectUDPSocketOptions,
    ErrorReason,
    ExemptedSetCookieWithReason,
    Headers,
    Initiator,
    InterceptionId,
    InterceptionStage,
    IPAddressSpace,
    LoaderId,
    LoadNetworkResourceOptions,
    LoadNetworkResourcePageResult,
    MonotonicTime,
    NetworkConditions,
    PostDataEntry,
    PrivateNetworkRequestPolicy,
    RenderBlockingBehavior,
    ReportId,
    ReportingApiEndpoint,
    ReportingApiReport,
    ReportStatus,
    Request,
    RequestId,
    RequestPattern,
    ResourcePriority,
    ResourceTiming,
    ResourceType,
    Response,
    SecurityDetails,
    SecurityIsolationStatus,
    ServiceWorkerResponseSource,
    ServiceWorkerRouterInfo,
    ServiceWorkerRouterSource,
    SetCookieBlockedReason,
    SignedCertificateTimestamp,
    SignedExchangeError,
    SignedExchangeErrorField,
    SignedExchangeHeader,
    SignedExchangeInfo,
    SignedExchangeSignature,
    TimeSinceEpoch,
    TrustTokenOperationType,
    TrustTokenParams,
    WebSocketFrame,
    WebSocketRequest,
    WebSocketResponse,
)

__all__ = [
    ResourceType,
    LoaderId,
    RequestId,
    InterceptionId,
    ErrorReason,
    TimeSinceEpoch,
    MonotonicTime,
    Headers,
    ConnectionType,
    CookieSameSite,
    CookiePriority,
    CookieSourceScheme,
    ResourceTiming,
    ResourcePriority,
    RenderBlockingBehavior,
    PostDataEntry,
    Request,
    SignedCertificateTimestamp,
    SecurityDetails,
    CertificateTransparencyCompliance,
    BlockedReason,
    CorsError,
    CorsErrorStatus,
    ServiceWorkerResponseSource,
    TrustTokenParams,
    TrustTokenOperationType,
    AlternateProtocolUsage,
    ServiceWorkerRouterSource,
    ServiceWorkerRouterInfo,
    Response,
    WebSocketRequest,
    WebSocketResponse,
    WebSocketFrame,
    CachedResource,
    Initiator,
    CookiePartitionKey,
    Cookie,
    SetCookieBlockedReason,
    CookieBlockedReason,
    CookieExemptionReason,
    BlockedSetCookieWithReason,
    ExemptedSetCookieWithReason,
    AssociatedCookie,
    CookieParam,
    AuthChallenge,
    AuthChallengeResponse,
    InterceptionStage,
    RequestPattern,
    SignedExchangeSignature,
    SignedExchangeHeader,
    SignedExchangeErrorField,
    SignedExchangeError,
    SignedExchangeInfo,
    ContentEncoding,
    NetworkConditions,
    BlockPattern,
    DirectSocketDnsQueryType,
    DirectTCPSocketOptions,
    DirectUDPSocketOptions,
    DirectUDPMessage,
    PrivateNetworkRequestPolicy,
    IPAddressSpace,
    ConnectTiming,
    ClientSecurityState,
    CrossOriginOpenerPolicyValue,
    CrossOriginOpenerPolicyStatus,
    CrossOriginEmbedderPolicyValue,
    CrossOriginEmbedderPolicyStatus,
    ContentSecurityPolicySource,
    ContentSecurityPolicyStatus,
    SecurityIsolationStatus,
    ReportStatus,
    ReportId,
    ReportingApiReport,
    ReportingApiEndpoint,
    LoadNetworkResourcePageResult,
    LoadNetworkResourceOptions,
    DataReceived,
    EventSourceMessageReceived,
    LoadingFailed,
    LoadingFinished,
    RequestIntercepted,
    RequestServedFromCache,
    RequestWillBeSent,
    ResourceChangedPriority,
    SignedExchangeReceived,
    ResponseReceived,
    WebSocketClosed,
    WebSocketCreated,
    WebSocketFrameError,
    WebSocketFrameReceived,
    WebSocketFrameSent,
    WebSocketHandshakeResponseReceived,
    WebSocketWillSendHandshakeRequest,
    WebTransportCreated,
    WebTransportConnectionEstablished,
    WebTransportClosed,
    DirectTCPSocketCreated,
    DirectTCPSocketOpened,
    DirectTCPSocketAborted,
    DirectTCPSocketClosed,
    DirectTCPSocketChunkSent,
    DirectTCPSocketChunkReceived,
    DirectUDPSocketJoinedMulticastGroup,
    DirectUDPSocketLeftMulticastGroup,
    DirectUDPSocketCreated,
    DirectUDPSocketOpened,
    DirectUDPSocketAborted,
    DirectUDPSocketClosed,
    DirectUDPSocketChunkSent,
    DirectUDPSocketChunkReceived,
    RequestWillBeSentExtraInfo,
    ResponseReceivedExtraInfo,
    ResponseReceivedEarlyHints,
    TrustTokenOperationDone,
    PolicyUpdated,
    ReportingApiReportAdded,
    ReportingApiReportUpdated,
    ReportingApiEndpointsChangedForOrigin,
    SetAcceptedEncodings,
    ClearAcceptedEncodingsOverride,
    CanClearBrowserCache,
    CanClearBrowserCookies,
    CanEmulateNetworkConditions,
    ClearBrowserCache,
    ClearBrowserCookies,
    ContinueInterceptedRequest,
    DeleteCookies,
    Disable,
    EmulateNetworkConditions,
    EmulateNetworkConditionsByRule,
    OverrideNetworkState,
    Enable,
    ConfigureDurableMessages,
    GetAllCookies,
    GetCertificate,
    GetCookies,
    GetResponseBody,
    GetRequestPostData,
    GetResponseBodyForInterception,
    TakeResponseBodyForInterceptionAsStream,
    ReplayXHR,
    SearchInResponseBody,
    SetBlockedURLs,
    SetBypassServiceWorker,
    SetCacheDisabled,
    SetCookie,
    SetCookies,
    SetExtraHTTPHeaders,
    SetAttachDebugStack,
    SetRequestInterception,
    SetUserAgentOverride,
    StreamResourceContent,
    GetSecurityIsolationStatus,
    EnableReportingApi,
    LoadNetworkResource,
    SetCookieControls,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Defines commands and events for browser extensions.
https://chromedevtools.github.io/devtools-protocol/tot/Extensions/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Extensions,
)
from cdpkit.protocol.base import JSON_DICT, CDPMethod, InputModel, OutputModel


class LoadUnpackedInput(InputModel):

    path: str


class LoadUnpackedOutput(OutputModel):

    id: str


class LoadUnpacked(CDPMethod[LoadUnpackedOutput]):
    """ Installs an unpacked extension from the filesystem similar to
    --load-extension CLI flags. Returns extension ID once the extension
    has been installed. Available if the client is connected using the
    --remote-debugging-pipe flag and the --enable-unsafe-extension-debugging
    flag is set. """

    INPUT_VALIDATOR = LoadUnpackedInput
    OUTPUT_VALIDATOR = LoadUnpackedOutput

    def __init__(
        self,
        *,
        path: str
    ):
        super().__init__(
            path=path
        )


class UninstallInput(InputModel):

    id: str


class Uninstall(CDPMethod[None]):
    """ Uninstalls an unpacked extension (others not supported) from the profile.
    Available if the client is connected using the --remote-debugging-pipe flag
    and the --enable-unsafe-extension-debugging. """

    INPUT_VALIDATOR = UninstallInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        id_: str
    ):
        super().__init__(
            id=id_
        )


class GetStorageItemsInput(InputModel):

    id: str
    storageArea: Extensions.StorageArea
    keys: list[str] | None = None


class GetStorageItemsOutput(OutputModel):

    data: JSON_DICT


class GetStorageItems(CDPMethod[GetStorageItemsOutput]):
    """ Gets data from extension storage in the given `storageArea`. If `keys` is
    specified, these are used to filter the result. """

    INPUT_VALIDATOR = GetStorageItemsInput
    OUTPUT_VALIDATOR = GetStorageItemsOutput

    def __init__(
        self,
        *,
        id_: str,
        storage_area: Extensions.StorageArea,
        keys: list[str] | None = None
    ):
        super().__init__(
            id=id_,
            storageArea=storage_area,
            keys=keys
        )


class RemoveStorageItemsInput(InputModel):

    id: str
    storageArea: Extensions.StorageArea
    keys: list[str]


class RemoveStorageItems(CDPMethod[None]):
    """ Removes `keys` from extension storage in the given `storageArea`. """

    INPUT_VALIDATOR = RemoveStorageItemsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        id_: str,
        storage_area: Extensions.StorageArea,
        keys: list[str]
    ):
        super().__init__(
            id=id_,
            storageArea=storage_area,
            keys=keys
        )


class ClearStorageItemsInput(InputModel):

    id: str
    storageArea: Extensions.StorageArea


class ClearStorageItems(CDPMethod[None]):
    """ Clears extension storage in the given `storageArea`. """

    INPUT_VALIDATOR = ClearStorageItemsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        id_: str,
        storage_area: Extensions.StorageArea
    ):
        super().__init__(
            id=id_,
            storageArea=storage_area
        )


class SetStorageItemsInput(InputModel):

    id: str
    storageArea: Extensions.StorageArea
    values: JSON_DICT


class SetStorageItems(CDPMethod[None]):
    """ Sets `values` in extension storage in the given `storageArea`. The provided `values`
    will be merged with existing values in the storage area. """

    INPUT_VALIDATOR = SetStorageItemsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        id_: str,
        storage_area: Extensions.StorageArea,
        values: JSON_DICT
    ):
        super().__init__(
            id=id_,
            storageArea=storage_area,
            values=values
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain emulates different environments for the page.
https://chromedevtools.github.io/devtools-protocol/tot/Emulation/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    EmulationDevicePosture,
    EmulationDisabledImageType,
    EmulationDisplayFeature,
    EmulationMediaFeature,
    EmulationPressureMetadata,
    EmulationPressureSource,
    EmulationPressureState,
    EmulationSafeAreaInsets,
    EmulationScreenId,
    EmulationScreenInfo,
    EmulationScreenOrientation,
    EmulationSensorMetadata,
    EmulationSensorReading,
    EmulationSensorReadingQuaternion,
    EmulationSensorReadingSingle,
    EmulationSensorReadingXYZ,
    EmulationSensorType,
    EmulationUserAgentBrandVersion,
    EmulationUserAgentMetadata,
    EmulationVirtualTimePolicy,
    EmulationWorkAreaInsets,
)

SafeAreaInsets = EmulationSafeAreaInsets
ScreenOrientation = EmulationScreenOrientation
DisplayFeature = EmulationDisplayFeature
DevicePosture = EmulationDevicePosture
MediaFeature = EmulationMediaFeature
VirtualTimePolicy = EmulationVirtualTimePolicy
UserAgentBrandVersion = EmulationUserAgentBrandVersion
UserAgentMetadata = EmulationUserAgentMetadata
SensorType = EmulationSensorType
SensorMetadata = EmulationSensorMetadata
SensorReadingSingle = EmulationSensorReadingSingle
SensorReadingXYZ = EmulationSensorReadingXYZ
SensorReadingQuaternion = EmulationSensorReadingQuaternion
SensorReading = EmulationSensorReading
PressureSource = EmulationPressureSource
PressureState = EmulationPressureState
PressureMetadata = EmulationPressureMetadata
WorkAreaInsets = EmulationWorkAreaInsets
ScreenId = EmulationScreenId
ScreenInfo = EmulationScreenInfo
DisabledImageType = EmulationDisabledImageType



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Query and modify DOM storage.
https://chromedevtools.github.io/devtools-protocol/tot/DOMStorage/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOMStorage,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class ClearInput(InputModel):

    storageId: DOMStorage.StorageId


class Clear(CDPMethod[None]):

    INPUT_VALIDATOR = ClearInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        storage_id: DOMStorage.StorageId
    ):
        super().__init__(
            storageId=storage_id
        )


class Disable(CDPMethod[None]):
    """ Disables storage tracking, prevents storage events from being sent to the client. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables storage tracking, storage events will now be delivered to the client. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetDOMStorageItemsInput(InputModel):

    storageId: DOMStorage.StorageId


class GetDOMStorageItemsOutput(OutputModel):

    entries: list[DOMStorage.Item]


class GetDOMStorageItems(CDPMethod[GetDOMStorageItemsOutput]):

    INPUT_VALIDATOR = GetDOMStorageItemsInput
    OUTPUT_VALIDATOR = GetDOMStorageItemsOutput

    def __init__(
        self,
        *,
        storage_id: DOMStorage.StorageId
    ):
        super().__init__(
            storageId=storage_id
        )


class RemoveDOMStorageItemInput(InputModel):

    storageId: DOMStorage.StorageId
    key: str


class RemoveDOMStorageItem(CDPMethod[None]):

    INPUT_VALIDATOR = RemoveDOMStorageItemInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        storage_id: DOMStorage.StorageId,
        key: str
    ):
        super().__init__(
            storageId=storage_id,
            key=key
        )


class SetDOMStorageItemInput(InputModel):

    storageId: DOMStorage.StorageId
    key: str
    value: str


class SetDOMStorageItem(CDPMethod[None]):

    INPUT_VALIDATOR = SetDOMStorageItemInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        storage_id: DOMStorage.StorageId,
        key: str,
        value: str
    ):
        super().__init__(
            storageId=storage_id,
            key=key,
            value=value
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain provides experimental commands only supported in headless mode.
https://chromedevtools.github.io/devtools-protocol/tot/HeadlessExperimental/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    HeadlessExperimental,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class BeginFrameInput(InputModel):

    frameTimeTicks: float | None = None
    interval: float | None = None
    noDisplayUpdates: bool | None = None
    screenshot: HeadlessExperimental.ScreenshotParams | None = None


class BeginFrameOutput(OutputModel):

    hasDamage: bool
    screenshotData: str | None = None


class BeginFrame(CDPMethod[BeginFrameOutput]):
    """ Sends a BeginFrame to the target and returns when the frame was completed. Optionally captures a
    screenshot from the resulting frame. Requires that the target was created with enabled
    BeginFrameControl. Designed for use with --run-all-compositor-stages-before-draw, see also
    https://goo.gle/chrome-headless-rendering for more background. """

    INPUT_VALIDATOR = BeginFrameInput
    OUTPUT_VALIDATOR = BeginFrameOutput

    def __init__(
        self,
        *,
        frame_time_ticks: float | None = None,
        interval: float | None = None,
        no_display_updates: bool | None = None,
        screenshot: HeadlessExperimental.ScreenshotParams | None = None
    ):
        super().__init__(
            frameTimeTicks=frame_time_ticks,
            interval=interval,
            noDisplayUpdates=no_display_updates,
            screenshot=screenshot
        )


class Disable(CDPMethod[None]):  # deprecated
    """ Disables headless events for the target. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):  # deprecated
    """ Enables headless events for the target. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



from .events import CertificateError, SecurityStateChanged, VisibleSecurityStateChanged
from .methods import Disable, Enable, HandleCertificateError, SetIgnoreCertificateErrors, SetOverrideCertificateErrors
from .types import (
    CertificateErrorAction,
    CertificateId,
    CertificateSecurityState,
    InsecureContentStatus,
    MixedContentType,
    SafetyTipInfo,
    SafetyTipStatus,
    SecurityState,
    SecurityStateExplanation,
    VisibleSecurityState,
)

__all__ = [
    CertificateId,
    MixedContentType,
    SecurityState,
    CertificateSecurityState,
    SafetyTipStatus,
    SafetyTipInfo,
    VisibleSecurityState,
    SecurityStateExplanation,
    InsecureContentStatus,
    CertificateErrorAction,
    CertificateError,
    VisibleSecurityStateChanged,
    SecurityStateChanged,
    Disable,
    Enable,
    SetIgnoreCertificateErrors,
    HandleCertificateError,
    SetOverrideCertificateErrors,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain exposes CSS read/write operations. All CSS objects (stylesheets, rules, and styles)
have an associated `id` used in subsequent operations on the related object. Each object type has
a specific `id` structure, and those are not interchangeable between objects of different kinds.
CSS objects can be loaded using the `get*ForNode()` calls (which accept a DOM node id). A client
can also keep track of stylesheets via the `styleSheetAdded`/`styleSheetRemoved` events and
subsequently load the required stylesheet contents using the `getStyleSheet[Text]()` methods.
https://chromedevtools.github.io/devtools-protocol/tot/CSS/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    CSS,
    DOM,
)
from cdpkit.protocol.base import CDPEvent


class FontsUpdated(CDPEvent):
    """ Fires whenever a web font is updated.  A non-empty font parameter indicates a successfully loaded
    web font. """

    font: CSS.FontFace | None = None


class MediaQueryResultChanged(CDPEvent):
    """ Fires whenever a MediaQuery result changes (for example, after a browser window has been
    resized.) The current implementation considers only viewport-dependent media features. """

    ...


class StyleSheetAdded(CDPEvent):
    """ Fired whenever an active document stylesheet is added. """

    header: CSS.CSSStyleSheetHeader


class StyleSheetChanged(CDPEvent):
    """ Fired whenever a stylesheet is changed as a result of the client operation. """

    styleSheetId: DOM.StyleSheetId


class StyleSheetRemoved(CDPEvent):
    """ Fired whenever an active document stylesheet is removed. """

    styleSheetId: DOM.StyleSheetId


class ComputedStyleUpdated(CDPEvent):

    nodeId: DOM.NodeId



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows inspection of Web Audio API.
https://webaudio.github.io/web-audio-api/
https://chromedevtools.github.io/devtools-protocol/tot/WebAudio/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    WebAudio,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class Enable(CDPMethod[None]):
    """ Enables the WebAudio domain and starts sending context lifetime events. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Disable(CDPMethod[None]):
    """ Disables the WebAudio domain. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetRealtimeDataInput(InputModel):

    contextId: WebAudio.GraphObjectId


class GetRealtimeDataOutput(OutputModel):

    realtimeData: WebAudio.ContextRealtimeData


class GetRealtimeData(CDPMethod[GetRealtimeDataOutput]):
    """ Fetch the realtime data from the registered contexts. """

    INPUT_VALIDATOR = GetRealtimeDataInput
    OUTPUT_VALIDATOR = GetRealtimeDataOutput

    def __init__(
        self,
        *,
        context_id: WebAudio.GraphObjectId
    ):
        super().__init__(
            contextId=context_id
        )



from .methods import ClearStorageItems, GetStorageItems, LoadUnpacked, RemoveStorageItems, SetStorageItems, Uninstall
from .types import StorageArea

__all__ = [
    StorageArea,
    LoadUnpacked,
    Uninstall,
    GetStorageItems,
    RemoveStorageItems,
    ClearStorageItems,
    SetStorageItems,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/ServiceWorker/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    ServiceWorkerRegistrationID,
    ServiceWorkerServiceWorkerErrorMessage,
    ServiceWorkerServiceWorkerRegistration,
    ServiceWorkerServiceWorkerVersion,
    ServiceWorkerServiceWorkerVersionRunningStatus,
    ServiceWorkerServiceWorkerVersionStatus,
)

RegistrationID = ServiceWorkerRegistrationID
ServiceWorkerRegistration = ServiceWorkerServiceWorkerRegistration
ServiceWorkerVersionRunningStatus = ServiceWorkerServiceWorkerVersionRunningStatus
ServiceWorkerVersionStatus = ServiceWorkerServiceWorkerVersionStatus
ServiceWorkerVersion = ServiceWorkerServiceWorkerVersion
ServiceWorkerErrorMessage = ServiceWorkerServiceWorkerErrorMessage



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain provides various functionality related to drawing atop the inspected page.
https://chromedevtools.github.io/devtools-protocol/tot/Overlay/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    OverlayBoxStyle,
    OverlayColorFormat,
    OverlayContainerQueryContainerHighlightConfig,
    OverlayContainerQueryHighlightConfig,
    OverlayContrastAlgorithm,
    OverlayFlexContainerHighlightConfig,
    OverlayFlexItemHighlightConfig,
    OverlayFlexNodeHighlightConfig,
    OverlayGridHighlightConfig,
    OverlayGridNodeHighlightConfig,
    OverlayHighlightConfig,
    OverlayHingeConfig,
    OverlayInspectMode,
    OverlayIsolatedElementHighlightConfig,
    OverlayIsolationModeHighlightConfig,
    OverlayLineStyle,
    OverlayScrollSnapContainerHighlightConfig,
    OverlayScrollSnapHighlightConfig,
    OverlaySourceOrderConfig,
    OverlayWindowControlsOverlayConfig,
)

SourceOrderConfig = OverlaySourceOrderConfig
GridHighlightConfig = OverlayGridHighlightConfig
FlexContainerHighlightConfig = OverlayFlexContainerHighlightConfig
FlexItemHighlightConfig = OverlayFlexItemHighlightConfig
LineStyle = OverlayLineStyle
BoxStyle = OverlayBoxStyle
ContrastAlgorithm = OverlayContrastAlgorithm
HighlightConfig = OverlayHighlightConfig
ColorFormat = OverlayColorFormat
GridNodeHighlightConfig = OverlayGridNodeHighlightConfig
FlexNodeHighlightConfig = OverlayFlexNodeHighlightConfig
ScrollSnapContainerHighlightConfig = OverlayScrollSnapContainerHighlightConfig
ScrollSnapHighlightConfig = OverlayScrollSnapHighlightConfig
HingeConfig = OverlayHingeConfig
WindowControlsOverlayConfig = OverlayWindowControlsOverlayConfig
ContainerQueryHighlightConfig = OverlayContainerQueryHighlightConfig
ContainerQueryContainerHighlightConfig = OverlayContainerQueryContainerHighlightConfig
IsolatedElementHighlightConfig = OverlayIsolatedElementHighlightConfig
IsolationModeHighlightConfig = OverlayIsolationModeHighlightConfig
InspectMode = OverlayInspectMode



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Input/Output operations for streams produced by DevTools.
https://chromedevtools.github.io/devtools-protocol/tot/IO/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    IO,
    Runtime,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class CloseInput(InputModel):

    handle: IO.StreamHandle


class Close(CDPMethod[None]):
    """ Close the stream, discard any temporary backing storage. """

    INPUT_VALIDATOR = CloseInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        handle: IO.StreamHandle
    ):
        super().__init__(
            handle=handle
        )


class ReadInput(InputModel):

    handle: IO.StreamHandle
    offset: int | None = None
    size: int | None = None


class ReadOutput(OutputModel):

    base64Encoded: bool | None = None
    data: str
    eof: bool


class Read(CDPMethod[ReadOutput]):
    """ Read a chunk of the stream """

    INPUT_VALIDATOR = ReadInput
    OUTPUT_VALIDATOR = ReadOutput

    def __init__(
        self,
        *,
        handle: IO.StreamHandle,
        offset: int | None = None,
        size: int | None = None
    ):
        super().__init__(
            handle=handle,
            offset=offset,
            size=size
        )


class ResolveBlobInput(InputModel):

    objectId: Runtime.RemoteObjectId


class ResolveBlobOutput(OutputModel):

    uuid: str


class ResolveBlob(CDPMethod[ResolveBlobOutput]):
    """ Return UUID of Blob object specified by a remote object id. """

    INPUT_VALIDATOR = ResolveBlobInput
    OUTPUT_VALIDATOR = ResolveBlobOutput

    def __init__(
        self,
        *,
        object_id: Runtime.RemoteObjectId
    ):
        super().__init__(
            objectId=object_id
        )



from .methods import (
    ChangeAppUserSettings,
    GetOsAppState,
    Install,
    Launch,
    LaunchFilesInApp,
    OpenCurrentPageInApp,
    Uninstall,
)
from .types import DisplayMode, FileHandler, FileHandlerAccept

__all__ = [
    FileHandlerAccept,
    FileHandler,
    DisplayMode,
    GetOsAppState,
    Install,
    Uninstall,
    Launch,
    LaunchFilesInApp,
    OpenCurrentPageInApp,
    ChangeAppUserSettings,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

The SystemInfo domain defines methods and events for querying low-level system information.
https://chromedevtools.github.io/devtools-protocol/tot/SystemInfo/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations



import asyncio
import json
from typing import Any

from pydantic import BaseModel, PrivateAttr

from cdpkit.logger import logger


class CommandsManager(BaseModel):
    _pending_commands: dict[int, asyncio.Future] = PrivateAttr(default_factory=dict)
    _command_id: int = PrivateAttr(default=0)

    def create_command_future(self) -> tuple[int, asyncio.Future]:
        self._command_id += 1
        future = asyncio.Future()
        self._pending_commands[self._command_id] = future
        return self._command_id, future

    def remove_pending_command(self, response_id: int):
        if response_id in self._pending_commands:
            del self._pending_commands[response_id]

    def resolve_command(self, message: dict[str, Any]):
        response_id = message.get('id')
        if response_id in self._pending_commands:
            self._pending_commands[response_id].set_result(json.dumps(message))
            del self._pending_commands[response_id]
        else:
            logger.warning(f'No pending message can be resolve for id {response_id}')



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows detailed inspection of media elements.
https://chromedevtools.github.io/devtools-protocol/tot/Media/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    MediaPlayer,
    MediaPlayerError,
    MediaPlayerErrorSourceLocation,
    MediaPlayerEvent,
    MediaPlayerId,
    MediaPlayerMessage,
    MediaPlayerProperty,
    MediaTimestamp,
)

PlayerId = MediaPlayerId
Timestamp = MediaTimestamp
PlayerMessage = MediaPlayerMessage
PlayerProperty = MediaPlayerProperty
PlayerEvent = MediaPlayerEvent
PlayerErrorSourceLocation = MediaPlayerErrorSourceLocation
PlayerError = MediaPlayerError
Player = MediaPlayer



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows detailed inspection of media elements.
https://chromedevtools.github.io/devtools-protocol/tot/Media/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Media,
)
from cdpkit.protocol.base import CDPEvent


class PlayerPropertiesChanged(CDPEvent):
    """ This can be called multiple times, and can be used to set / override /
    remove player properties. A null propValue indicates removal. """

    playerId: Media.PlayerId
    properties: list[Media.PlayerProperty]


class PlayerEventsAdded(CDPEvent):
    """ Send events as a list, allowing them to be batched on the browser for less
    congestion. If batched, events must ALWAYS be in chronological order. """

    playerId: Media.PlayerId
    events: list[Media.PlayerEvent]


class PlayerMessagesLogged(CDPEvent):
    """ Send a list of any messages that need to be delivered. """

    playerId: Media.PlayerId
    messages: list[Media.PlayerMessage]


class PlayerErrorsRaised(CDPEvent):
    """ Send a list of any errors that need to be delivered. """

    playerId: Media.PlayerId
    errors: list[Media.PlayerError]


class PlayerCreated(CDPEvent):
    """ Called whenever a player is created, or when a new agent joins and receives
    a list of active players. If an agent is restored, it will receive one
    event for each active player. """

    player: Media.Player
